package eu.eudat.elastic.repository;

import eu.eudat.elastic.criteria.DatasetCriteria;
import eu.eudat.elastic.entities.Dataset;
import eu.eudat.elastic.entities.Tag;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.reindex.DeleteByQueryRequest;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service("datasetRepository")
public class DatasetRepository extends ElasticRepository<Dataset, DatasetCriteria> {

    @Autowired
    public DatasetRepository(RestHighLevelClient client) {
        super(client);
    }

    @Override
    public Dataset createOrUpdate(Dataset entity) throws IOException {
        XContentBuilder builder = XContentFactory.jsonBuilder();
        IndexRequest request = new IndexRequest("datasets").id(entity.getId()).source(entity.toElasticEntity(builder));//new IndexRequest("datasets", "doc", entity.getId()).source(entity.toElasticEntity(builder));
        this.getClient().index(request, RequestOptions.DEFAULT);
        return entity;
    }

    @Override
    public Dataset findDocument(String id) throws IOException {
        GetRequest request = new GetRequest("datasets",id);
        GetResponse response = this.getClient().get(request, RequestOptions.DEFAULT);
        return new Dataset().fromElasticEntity(response.getSourceAsMap());
    }

    @Override
    public List<Dataset> query(DatasetCriteria criteria) throws IOException {
        SearchRequest searchRequest = new SearchRequest("datasets");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

        CountRequest countRequest = new CountRequest("datasets");
        countRequest.query(QueryBuilders.boolQuery().mustNot(QueryBuilders.termsQuery("status.keyword", Stream.of(Dataset.Status.DELETED.getValue(), Dataset.Status.CANCELED.getValue()).collect(Collectors.toList()))));
        CountResponse countResponse = getClient().count(countRequest, RequestOptions.DEFAULT);
        Long count = countResponse.getCount();

        searchSourceBuilder.size(count.intValue());

        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery().mustNot(QueryBuilders.termsQuery("status.keyword", Stream.of(Dataset.Status.DELETED.getValue(), Dataset.Status.CANCELED.getValue()).collect(Collectors.toList())));
        if (criteria.isPublic()) {
            boolQuery = boolQuery.should(QueryBuilders.termQuery("public.keyword", "true"));
            boolQuery = boolQuery.should(QueryBuilders.termQuery("status.keyword", Dataset.Status.FINALISED.getValue()));
            boolQuery = boolQuery.should(QueryBuilders.termQuery("lastPublicVersion.keyword", "true"));
        }
        if (criteria.getLike() != null && !criteria.getLike().isEmpty()) {
            boolQuery = boolQuery.should(QueryBuilders.queryStringQuery(criteria.getLike()).fields(Stream.of(new Object[][] {
                    { "label", 1.0f },
                    { "description", 1.0f },
                    { "formData", 1.0f }
            }).collect(Collectors.toMap(data -> (String) data[0], data -> (Float) data[1]))));
        }

        if (criteria.getDatasetTemplates() != null && criteria.getDatasetTemplates().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("template.keyword", criteria.getDatasetTemplates().stream().map(UUID::toString).collect(Collectors.toList())));
        }

        if (criteria.getStatus() != null) {
            boolQuery = boolQuery.should(QueryBuilders.termQuery("status.keyword", criteria.getStatus().toString()));
        }

        if (criteria.getDmps() != null && criteria.getDmps().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("dmp.keyword", criteria.getDmps().stream().map(UUID::toString).collect(Collectors.toList())));
        }

        if (criteria.getGroupIds() != null && criteria.getGroupIds().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("group.keyword", criteria.getGroupIds().stream().map(UUID::toString).collect(Collectors.toList())));
        }

        if (criteria.getGrants() != null && criteria.getGrants().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("grant.keyword", criteria.getGrants().stream().map(UUID::toString).collect(Collectors.toList())));
        }

        if (criteria.getGrantStatus() != null) {
            boolQuery = boolQuery.should(QueryBuilders.termQuery("grantStatus.keyword", criteria.getGrantStatus().toString()));
        }

        if (criteria.getCollaborators() != null && criteria.getCollaborators().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("collaborators.id.keyword", criteria.getCollaborators().stream().map(UUID::toString).collect(Collectors.toList())));
        }

        if (!criteria.isPublic()) {
            if (criteria.getAllowAllVersions() != null && !criteria.getAllowAllVersions()) {
                boolQuery = boolQuery.should(QueryBuilders.termQuery("lastVersion.keyword", "true"));
            }
        }

        if (criteria.getOrganiztions() != null && criteria.getOrganiztions().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("organizations.id.keyword", criteria.getOrganiztions()));
        }

        if (criteria.getTags() != null && criteria.getTags().size() > 0) {
            boolQuery = boolQuery.should(QueryBuilders.termsQuery("tags.name.keyword", criteria.getTags().stream().map(Tag::getName).collect(Collectors.toList())));
        }

        if (boolQuery.should().isEmpty() && boolQuery.mustNot().isEmpty()) {
            boolQuery.should(QueryBuilders.matchAllQuery());
        } else {
            boolQuery.minimumShouldMatch(boolQuery.should().size());
        }
        searchSourceBuilder.query(boolQuery);
        searchRequest.source(searchSourceBuilder);
        SearchResponse response = this.getClient().search(searchRequest, RequestOptions.DEFAULT);
        return Arrays.stream(response.getHits().getHits()).map(x -> this.transformFromString(x.getSourceAsString(), Dataset.class)).collect(Collectors.toList());
    }

    @Override
    public boolean exists() throws IOException {
        GetIndexRequest request = new GetIndexRequest("datasets");
//        request.indices("datasets");
        return this.getClient().indices().exists(request, RequestOptions.DEFAULT);
    }

    @Override
    public void clear() throws IOException {
        if (exists()) {
            DeleteByQueryRequest delete = new DeleteByQueryRequest("datasets");
            delete.setQuery(QueryBuilders.matchAllQuery());
            this.getClient().deleteByQuery(delete, RequestOptions.DEFAULT);
        }
    }
}
